#ifndef POKEPLATINUM_UNK_0200112C_H
#define POKEPLATINUM_UNK_0200112C_H

#include "struct_decls/struct_0200112C_decl.h"
#include "struct_decls/struct_02013A04_decl.h"
#include "overlay084/struct_ov84_02240FA8.h"

UnkStruct_0200112C * sub_0200112C(const UnkStruct_ov84_02240FA8 * param0, u16 param1, u16 param2, u8 param3);
u32 sub_02001288(UnkStruct_0200112C * param0);
void sub_02001384(UnkStruct_0200112C * param0, u16 * param1, u16 * param2);
void sub_020013AC(UnkStruct_0200112C * param0);
void sub_020013D8(UnkStruct_0200112C * param0, u8 param1, u8 param2, u8 param3);
u32 sub_02001408(UnkStruct_0200112C * param0, UnkStruct_ov84_02240FA8 * param1, u16 param2, u16 param3, u16 param4, u16 param5, u16 * param6, u16 * param7);
void sub_0200147C(UnkStruct_0200112C * param0, u8 param1, u8 param2, u8 param3);
void sub_020014D0(UnkStruct_0200112C * param0, u16 * param1);
void sub_020014DC(UnkStruct_0200112C * param0, u16 * param1, u16 * param2);
u8 sub_020014F0(UnkStruct_0200112C * param0);
u32 sub_020014F8(UnkStruct_0200112C * param0, u16 param1);
u32 sub_02001504(UnkStruct_0200112C * param0, u8 param1);
void sub_020015CC(UnkStruct_0200112C * param0, UnkStruct_02013A04 * param1);

#endif // POKEPLATINUM_UNK_0200112C_H
